package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.ReviewHistoryEntry;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.repository.jpa.ReviewHistoryEntryJPARepository;
import at.ac.uibk.gitsearch.service.dto.ReviewHistoryEntryDTO;
import at.ac.uibk.gitsearch.service.mapper.ReviewHistoryEntryMapper;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service Implementation for managing {@link ReviewHistoryEntry}.
 */
@Service
@Transactional
public class ReviewHistoryEntryService {

    private final Logger log = LoggerFactory.getLogger(ReviewHistoryEntryService.class);

    private final ReviewHistoryEntryJPARepository reviewHistoryEntryRepository;

    private final ReviewHistoryEntryMapper reviewHistoryEntryMapper;

    private final UserService userService;

    public ReviewHistoryEntryService(
        ReviewHistoryEntryJPARepository reviewHistoryEntryRepository,
        ReviewHistoryEntryMapper reviewHistoryEntryMapper,
        UserService userService
    ) {
        this.reviewHistoryEntryRepository = reviewHistoryEntryRepository;
        this.reviewHistoryEntryMapper = reviewHistoryEntryMapper;
        this.userService = userService;
    }

    /**
     * Save a reviewHistoryEntry.
     *
     * @param reviewHistoryEntryDTO the entity to save.
     * @return the persisted entity.
     */
    public ReviewHistoryEntryDTO save(ReviewHistoryEntryDTO reviewHistoryEntryDTO) {
        log.debug("Request to save ReviewHistoryEntry : {}", reviewHistoryEntryDTO);
        ReviewHistoryEntry reviewHistoryEntry = reviewHistoryEntryMapper.toEntity(reviewHistoryEntryDTO);
        reviewHistoryEntry = reviewHistoryEntryRepository.save(reviewHistoryEntry);
        return reviewHistoryEntryMapper.toDto(reviewHistoryEntry);
    }

    /**
     * Partially update a reviewHistoryEntry.
     *
     * @param reviewHistoryEntryDTO the entity to update partially.
     * @return the persisted entity.
     */
    public Optional<ReviewHistoryEntryDTO> partialUpdate(ReviewHistoryEntryDTO reviewHistoryEntryDTO) {
        log.debug("Request to partially update ReviewHistoryEntry : {}", reviewHistoryEntryDTO);

        return reviewHistoryEntryRepository
            .findById(reviewHistoryEntryDTO.getId())
            .map(existingReviewHistoryEntry -> {
                reviewHistoryEntryMapper.partialUpdate(existingReviewHistoryEntry, reviewHistoryEntryDTO);

                return existingReviewHistoryEntry;
            })
            .map(reviewHistoryEntryRepository::save)
            .map(reviewHistoryEntryMapper::toDto);
    }

    /**
     * Get all the reviewHistoryEntries.
     *
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public List<ReviewHistoryEntryDTO> findAll() {
        log.debug("Request to get all ReviewHistoryEntries");
        return reviewHistoryEntryRepository
            .findAll()
            .stream()
            .map(reviewHistoryEntryMapper::toDto)
            .collect(Collectors.toCollection(LinkedList::new));
    }

    /**
     * Get reviewHistoryEntries for user.
     *
     * @param reviewId       the id of the review
     * @param reviewRatingId the id of the review rating
     * @param user           the user
     *
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public List<ReviewHistoryEntryDTO> findByReviewReviewRating(Long reviewId, Long reviewRatingId) {
        final Optional<User> userO = userService.getUserWithAuthorities();
        if (userO.isEmpty()) {
            log.warn("The method findByReviewReviewRating may not called without current user");
            return new ArrayList<>();
        }
        log.debug("Request to get ReviewHistoryEntries for user");
        return reviewHistoryEntryRepository
            .findByReviewReviewRatingAndUser(reviewId, reviewRatingId, userO.get())
            .stream()
            .map(reviewHistoryEntryMapper::toDto)
            .collect(Collectors.toCollection(LinkedList::new));
    }

    /**
     * Get one reviewHistoryEntry by id.
     *
     * @param id the id of the entity.
     * @return the entity.
     */
    @Transactional(readOnly = true)
    public Optional<ReviewHistoryEntryDTO> findOne(Long id) {
        log.debug("Request to get ReviewHistoryEntry : {}", id);
        return reviewHistoryEntryRepository.findById(id).map(reviewHistoryEntryMapper::toDto);
    }

    /**
     * Delete the reviewHistoryEntry by id.
     *
     * @param id the id of the entity.
     */
    public void delete(Long id) {
        log.debug("Request to delete ReviewHistoryEntry : {}", id);
        reviewHistoryEntryRepository.deleteById(id);
    }
}
