package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.Statistics;
import at.ac.uibk.gitsearch.repository.jpa.StatisticsRepository;
import at.ac.uibk.gitsearch.repository.search.ElasticSearchRepository;
import at.ac.uibk.gitsearch.service.dto.StatisticsDTO;
import java.io.IOException;
import java.util.List;
import java.util.Optional;
import org.codeability.sharing.plugins.api.search.SearchResultDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

/**
 * Service Interface for managing {@link at.ac.uibk.gitsearch.domain.Statistics}.
 */
@Service
public class StatisticsService {

    private final Logger log = LoggerFactory.getLogger(StatisticsService.class);

    @Autowired
    private StatisticsRepository statisticsRepository;

    @Autowired
    private ElasticSearchRepository elasticSearchRepository;

    /**
     * Save a statistics.
     *
     * @param statisticsDTO the entity to save.
     * @return the persisted entity.
     */
    public StatisticsDTO save(StatisticsDTO statisticsDTO) {
        Statistics statistics = new Statistics();
        statistics.setId(statisticsDTO.getId());
        statistics.setExerciseID(statisticsDTO.getExerciseID());
        statistics.setDownloads(statisticsDTO.getDownloads());
        statistics.setViews(statisticsDTO.getViews());
        statistics.setBadgeRewarded(statisticsDTO.getBadgeRewarded());
        statisticsRepository.save(statistics);

        try {
            final SearchResultDTO searchResultDTO = elasticSearchRepository.get(statisticsDTO.getExerciseID());
            if (searchResultDTO != null) {
                elasticSearchRepository.updateSearchStatistics(searchResultDTO, statisticsDTO);
                elasticSearchRepository.index(searchResultDTO);
            }
        } catch (IOException e) {
            log.error("Cannot update statistics for {}", statisticsDTO.getExerciseID(), e);
        }
        return statisticsDTO;
    }

    /**
     * Get all the statistics.
     *
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    public Page<StatisticsDTO> findAll(Pageable pageable) {
        return statisticsRepository.findAll(pageable).map(StatisticsDTO::fromEntity);
    }

    /**
     * Get the "id" statistics.
     *
     * @param id the id of the entity.
     * @return the entity.
     */
    public Optional<StatisticsDTO> findOne(Long id) {
        log.debug("Request to get Statistics : {}", id);
        return statisticsRepository.findById(id).map(StatisticsDTO::fromEntity);
    }

    public Optional<StatisticsDTO> findOneByExerciseID(String id) {
        log.debug("Request to get Statistics : {}", id);
        return statisticsRepository.findByExerciseID(id).map(StatisticsDTO::fromEntity);
    }

    public void rewardBadgeForStatisticsByExerciseID(String exerciseID) {
        Optional<Statistics> statistics = statisticsRepository.findByExerciseID(exerciseID);
        if (statistics.isPresent()) {
            Statistics statistics1 = statistics.get();
            statistics1.setBadgeRewarded(true);
            statisticsRepository.save(statistics1);
            log.debug("Badge rewarded for exercise: {}", exerciseID);
        } else {
            log.debug("No statistics found for exercise: {}", exerciseID);
            Statistics statistics1 = new Statistics();
            statistics1.setExerciseID(exerciseID);
            statistics1.setBadgeRewarded(true);
            statistics1.setDownloads(0);
            statistics1.setViews(0);
            statisticsRepository.save(statistics1);
        }
    }

    public void removeBadgeForStatisticsByExerciseID(String exerciseID) {
        Optional<Statistics> statistics = statisticsRepository.findByExerciseID(exerciseID);
        if (statistics.isPresent()) {
            Statistics statistics1 = statistics.get();
            statistics1.setBadgeRewarded(false);
            statisticsRepository.save(statistics1);
            log.debug("Badge removed for exercise: {}", exerciseID);
        } else {
            log.debug("No statistics found for exercise: {}", exerciseID);
            Statistics statistics1 = new Statistics();
            statistics1.setExerciseID(exerciseID);
            statistics1.setBadgeRewarded(false);
            statistics1.setDownloads(0);
            statistics1.setViews(0);
            statisticsRepository.save(statistics1);
        }
    }

    /**
     * Delete the "id" statistics.
     *
     * @param id the id of the entity.
     */
    public void delete(Long id) {
        log.debug("Request to delete Statistics : {}", id);
        statisticsRepository.deleteById(id);
    }

    /**
     * returns the sum of all downloads and views of all entries.
     *
     * @return the two values of downlodas and views (in this sequence)
     */
    List<Integer[]> findNumberOfDownloadsAndViews() {
        return statisticsRepository.findNumberOfDownloadsAndViews();
    }
}
