import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { of, Subject, from } from 'rxjs';

import { ReviewHistoryEntryService } from '../service/review-history-entry.service';
import { IReviewHistoryEntry, ReviewHistoryEntry } from '../review-history-entry.model';
import { IReview } from 'app/entities/review/review.model';
import { ReviewService } from 'app/entities/review/service/review.service';
import { IReviewRating } from 'app/entities/review-rating/review-rating.model';
import { ReviewRatingService } from 'app/entities/review-rating/service/review-rating.service';

import { IUser } from 'app/entities/user/user.model';
import { UserService } from 'app/entities/user/user.service';

import { ReviewHistoryEntryUpdateComponent } from './review-history-entry-update.component';

describe('ReviewHistoryEntry Management Update Component', () => {
  let comp: ReviewHistoryEntryUpdateComponent;
  let fixture: ComponentFixture<ReviewHistoryEntryUpdateComponent>;
  let activatedRoute: ActivatedRoute;
  let reviewHistoryEntryService: ReviewHistoryEntryService;
  let reviewService: ReviewService;
  let reviewRatingService: ReviewRatingService;
  let userService: UserService;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule, RouterTestingModule.withRoutes([])],
      declarations: [ReviewHistoryEntryUpdateComponent],
      providers: [
        FormBuilder,
        {
          provide: ActivatedRoute,
          useValue: {
            params: from([{}]),
          },
        },
      ],
    })
      .overrideTemplate(ReviewHistoryEntryUpdateComponent, '')
      .compileComponents();

    fixture = TestBed.createComponent(ReviewHistoryEntryUpdateComponent);
    activatedRoute = TestBed.inject(ActivatedRoute);
    reviewHistoryEntryService = TestBed.inject(ReviewHistoryEntryService);
    reviewService = TestBed.inject(ReviewService);
    reviewRatingService = TestBed.inject(ReviewRatingService);
    userService = TestBed.inject(UserService);

    comp = fixture.componentInstance;
  });

  describe('ngOnInit', () => {
    it('Should call Review query and add missing value', () => {
      const reviewHistoryEntry: IReviewHistoryEntry = { id: 456 };
      const review: IReview = { id: 63853 };
      reviewHistoryEntry.review = review;

      const reviewCollection: IReview[] = [{ id: 37679 }];
      jest.spyOn(reviewService, 'query').mockReturnValue(of(new HttpResponse({ body: reviewCollection })));
      const additionalReviews = [review];
      const expectedCollection: IReview[] = [...additionalReviews, ...reviewCollection];
      jest.spyOn(reviewService, 'addReviewToCollectionIfMissing').mockReturnValue(expectedCollection);

      activatedRoute.data = of({ reviewHistoryEntry });
      comp.ngOnInit();

      expect(reviewService.query).toHaveBeenCalled();
      expect(reviewService.addReviewToCollectionIfMissing).toHaveBeenCalledWith(reviewCollection, ...additionalReviews);
      expect(comp.reviewsSharedCollection).toEqual(expectedCollection);
    });

    it('Should call ReviewRating query and add missing value', () => {
      const reviewHistoryEntry: IReviewHistoryEntry = { id: 456 };
      const reviewRating: IReviewRating = { id: 47665 };
      reviewHistoryEntry.reviewRating = reviewRating;

      const reviewRatingCollection: IReviewRating[] = [{ id: 82705 }];
      jest.spyOn(reviewRatingService, 'query').mockReturnValue(of(new HttpResponse({ body: reviewRatingCollection })));
      const additionalReviewRatings = [reviewRating];
      const expectedCollection: IReviewRating[] = [...additionalReviewRatings, ...reviewRatingCollection];
      jest.spyOn(reviewRatingService, 'addReviewRatingToCollectionIfMissing').mockReturnValue(expectedCollection);

      activatedRoute.data = of({ reviewHistoryEntry });
      comp.ngOnInit();

      expect(reviewRatingService.query).toHaveBeenCalled();
      expect(reviewRatingService.addReviewRatingToCollectionIfMissing).toHaveBeenCalledWith(
        reviewRatingCollection,
        ...additionalReviewRatings
      );
      expect(comp.reviewRatingsSharedCollection).toEqual(expectedCollection);
    });

    it('Should call User query and add missing value', () => {
      const reviewHistoryEntry: IReviewHistoryEntry = { id: 456 };
      const user: IUser = { id: 43887 };
      reviewHistoryEntry.user = user;

      const userCollection: IUser[] = [{ id: 35212 }];
      jest.spyOn(userService, 'query').mockReturnValue(of(new HttpResponse({ body: userCollection })));
      const additionalUsers = [user];
      const expectedCollection: IUser[] = [...additionalUsers, ...userCollection];
      jest.spyOn(userService, 'addUserToCollectionIfMissing').mockReturnValue(expectedCollection);

      activatedRoute.data = of({ reviewHistoryEntry });
      comp.ngOnInit();

      expect(userService.query).toHaveBeenCalled();
      expect(userService.addUserToCollectionIfMissing).toHaveBeenCalledWith(userCollection, ...additionalUsers);
      expect(comp.usersSharedCollection).toEqual(expectedCollection);
    });

    it('Should update editForm', () => {
      const reviewHistoryEntry: IReviewHistoryEntry = { id: 456 };
      const review: IReview = { id: 33448 };
      reviewHistoryEntry.review = review;
      const reviewRating: IReviewRating = { id: 37417 };
      reviewHistoryEntry.reviewRating = reviewRating;
      const user: IUser = { id: 34182 };
      reviewHistoryEntry.user = user;

      activatedRoute.data = of({ reviewHistoryEntry });
      comp.ngOnInit();

      expect(comp.editForm.value).toEqual(expect.objectContaining(reviewHistoryEntry));
      expect(comp.reviewsSharedCollection).toContain(review);
      expect(comp.reviewRatingsSharedCollection).toContain(reviewRating);
      expect(comp.usersSharedCollection).toContain(user);
    });
  });

  describe('save', () => {
    it('Should call update service on save for existing entity', () => {
      // GIVEN
      const saveSubject = new Subject<HttpResponse<ReviewHistoryEntry>>();
      const reviewHistoryEntry = { id: 123 };
      jest.spyOn(reviewHistoryEntryService, 'update').mockReturnValue(saveSubject);
      jest.spyOn(comp, 'previousState');
      activatedRoute.data = of({ reviewHistoryEntry });
      comp.ngOnInit();

      // WHEN
      comp.save();
      expect(comp.isSaving).toEqual(true);
      saveSubject.next(new HttpResponse({ body: reviewHistoryEntry }));
      saveSubject.complete();

      // THEN
      expect(comp.previousState).toHaveBeenCalled();
      expect(reviewHistoryEntryService.update).toHaveBeenCalledWith(reviewHistoryEntry);
      expect(comp.isSaving).toEqual(false);
    });

    it('Should call create service on save for new entity', () => {
      // GIVEN
      const saveSubject = new Subject<HttpResponse<ReviewHistoryEntry>>();
      const reviewHistoryEntry = new ReviewHistoryEntry();
      jest.spyOn(reviewHistoryEntryService, 'create').mockReturnValue(saveSubject);
      jest.spyOn(comp, 'previousState');
      activatedRoute.data = of({ reviewHistoryEntry });
      comp.ngOnInit();

      // WHEN
      comp.save();
      expect(comp.isSaving).toEqual(true);
      saveSubject.next(new HttpResponse({ body: reviewHistoryEntry }));
      saveSubject.complete();

      // THEN
      expect(reviewHistoryEntryService.create).toHaveBeenCalledWith(reviewHistoryEntry);
      expect(comp.isSaving).toEqual(false);
      expect(comp.previousState).toHaveBeenCalled();
    });

    it('Should set isSaving to false on error', () => {
      // GIVEN
      const saveSubject = new Subject<HttpResponse<ReviewHistoryEntry>>();
      const reviewHistoryEntry = { id: 123 };
      jest.spyOn(reviewHistoryEntryService, 'update').mockReturnValue(saveSubject);
      jest.spyOn(comp, 'previousState');
      activatedRoute.data = of({ reviewHistoryEntry });
      comp.ngOnInit();

      // WHEN
      comp.save();
      expect(comp.isSaving).toEqual(true);
      saveSubject.error('This is an error!');

      // THEN
      expect(reviewHistoryEntryService.update).toHaveBeenCalledWith(reviewHistoryEntry);
      expect(comp.isSaving).toEqual(false);
      expect(comp.previousState).not.toHaveBeenCalled();
    });
  });

  describe('Tracking relationships identifiers', () => {
    describe('trackReviewById', () => {
      it('Should return tracked Review primary key', () => {
        const entity = { id: 123 };
        const trackResult = comp.trackReviewById(0, entity);
        expect(trackResult).toEqual(entity.id);
      });
    });

    describe('trackReviewRatingById', () => {
      it('Should return tracked ReviewRating primary key', () => {
        const entity = { id: 123 };
        const trackResult = comp.trackReviewRatingById(0, entity);
        expect(trackResult).toEqual(entity.id);
      });
    });

    describe('trackUserById', () => {
      it('Should return tracked User primary key', () => {
        const entity = { id: 123 };
        const trackResult = comp.trackUserById(0, entity);
        expect(trackResult).toEqual(entity.id);
      });
    });
  });
});
